@@ -1,5 +1,5 @@
 // libsemigroups - C++ library for semigroups and monoids
-// Copyright (C) 2021-2023 James D. Mitchell + Maria Tsalakou
+// Copyright (C) 2021-2025 James D. Mitchell + Maria Tsalakou
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,30 +21,25 @@
 #include <utility>    // for pair
 #include <vector>     // for vector
 
-#include "catch.hpp"      // for REQUIRE, REQUIRE_THROWS_AS
-#include "test-main.hpp"  // for LIBSEMIGROUPS_TEST_CASE
+#include "Catch2-3.8.0/catch_amalgamated.hpp"  // for REQUIRE, REQUIRE_THROWS_AS
+#include "test-main.hpp"                       // for LIBSEMIGROUPS_TEST_CASE
 
-#include "libsemigroups/constants.hpp"  // for operator==, POSITIVE_INFINITY
-#include "libsemigroups/exception.hpp"  // for LIBSEMIGROUPS_EXCEPTION
-#include "libsemigroups/int-range.hpp"  // for IntegralRange
-#include "libsemigroups/types.hpp"      // for word_type
-#include "libsemigroups/ukkonen.hpp"    // for Ukkonen, Ukkonen::State
-#include "libsemigroups/wislo.hpp"      // for const_wislo_iterator, cbegi...
-#include "libsemigroups/word.hpp"       // for literals
+#include "libsemigroups/constants.hpp"   // for operator==, POSITIVE_INFINITY
+#include "libsemigroups/exception.hpp"   // for LIBSEMIGROUPS_EXCEPTION
+#include "libsemigroups/types.hpp"       // for word_type
+#include "libsemigroups/ukkonen.hpp"     // for Ukkonen, Ukkonen::State
+#include "libsemigroups/word-range.hpp"  // for literals
+
+#include "libsemigroups/detail/int-range.hpp"  // for IntRange
 
 namespace libsemigroups {
 
   using literals::operator""_w;
 
-  namespace {
-    // TODO Should really be just a use of presentation::longest_common_subword
-    auto best_subword(Ukkonen& u) {
-      ukkonen::detail::GreedyReduceHelper helper(u);
-      return ukkonen::dfs(u, helper);
-    }
-  }  // namespace
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "000", "basic tests", "[quick][ukkonen]") {
+  LIBSEMIGROUPS_TEST_CASE("Ukkonen",
+                          "000",
+                          "basic tests",
+                          "[quick][ukkonen][no-valgrind]") {
     Ukkonen t;
     // aaeaaa$
     // abcd$'
@@ -53,7 +48,7 @@ namespace libsemigroups {
     // c -> 2
     // d -> 3
     // e -> 4
-    t.add_word(004000_w);
+    ukkonen::add_word(t, 004000_w);
     REQUIRE(t.nodes().size() == 10);
 
     REQUIRE(ukkonen::is_subword(t, 004000_w));
@@ -67,11 +62,12 @@ namespace libsemigroups {
     REQUIRE(!ukkonen::is_subword(t, 0000_w));
     REQUIRE(!ukkonen::is_subword(t, 1_w));
     REQUIRE(ukkonen::number_of_distinct_subwords(t) == 16);
-    REQUIRE(cbegin_wislo(5, {}, 0000000_w)->empty());
-    REQUIRE(std::count_if(
-                cbegin_wislo(6, {}, 00000000_w),
-                cend_wislo(6, {}, 00000000_w),
-                [&t](word_type const& w) { return ukkonen::is_subword(t, w); })
+    WordRange w;
+    w.alphabet_size(6).min(0).max(8);
+    REQUIRE((w | rx::filter([&t](word_type const& u) {
+               return ukkonen::is_subword(t, u);
+             })
+             | rx::count())
             == 16);
 
     REQUIRE(ukkonen::is_subword(t, ""_w));  // 1
@@ -91,7 +87,7 @@ namespace libsemigroups {
     REQUIRE(ukkonen::is_subword(t, 4_w));    // 15
     REQUIRE(ukkonen::is_subword(t, 000_w));  // 16
 
-    t.add_word(0123_w);
+    ukkonen::add_word(t, 0123_w);
     REQUIRE(t.nodes().size() == 15);
 
     REQUIRE(ukkonen::is_subword(t, ""_w));  // 1
@@ -155,10 +151,11 @@ namespace libsemigroups {
     REQUIRE(!ukkonen::is_suffix_no_checks(t, "ab"));
     REQUIRE(!ukkonen::is_suffix_no_checks(t, std::string("ab")));
 
-    REQUIRE(std::count_if(
-                cbegin_wislo(5, {}, 0000000_w),
-                cend_wislo(5, {}, 0000000_w),
-                [&t](word_type const& w) { return ukkonen::is_suffix(t, w); })
+    w.alphabet_size(5).min(0).max(7);
+    REQUIRE((w | rx::filter([&t](word_type const& u) {
+               return ukkonen::is_suffix(t, u);
+             })
+             | rx::count())
             == 11);
 
     REQUIRE(ukkonen::length_maximal_piece_prefix(t, 004000_w) == 2);
@@ -193,14 +190,14 @@ namespace libsemigroups {
                           "maximal_piece_prefix 1",
                           "[quick][ukkonen]") {
     Ukkonen t;
-    t.add_word({0, 5, 7});
-    t.add_word({1, 6, 7});
-    t.add_word({7, 2});
-    t.add_word({3, 4});
-    t.add_word({4, 8});
-    t.add_word({9});
-    t.add_word({5, 7, 10});
-    t.add_word({6, 7, 11});
+    ukkonen::add_word(t, {0, 5, 7});
+    ukkonen::add_word(t, {1, 6, 7});
+    ukkonen::add_word(t, {7, 2});
+    ukkonen::add_word(t, {3, 4});
+    ukkonen::add_word(t, {4, 8});
+    ukkonen::add_word(t, {9});
+    ukkonen::add_word(t, {5, 7, 10});
+    ukkonen::add_word(t, {6, 7, 11});
 
     REQUIRE(t.nodes().size() == 32);
     REQUIRE(ukkonen::length_maximal_piece_prefix(t, {0, 5, 7}) == 0);
@@ -218,20 +215,20 @@ namespace libsemigroups {
                           "maximal_piece_prefix 2",
                           "[quick][ukkonen]") {
     Ukkonen t;
-    t.add_word(004000_w);
-    t.add_word(45_w);
+    ukkonen::add_word(t, 004000_w);
+    ukkonen::add_word(t, 45_w);
 
     REQUIRE(ukkonen::number_of_distinct_subwords(t) == 18);
     REQUIRE(ukkonen::length_maximal_piece_prefix(t, 004000_w) == 2);
     REQUIRE(ukkonen::length_maximal_piece_prefix(t, 45_w) == 1);
 
-    t.add_word(0123_w);
+    ukkonen::add_word(t, 0123_w);
     REQUIRE(ukkonen::number_of_distinct_subwords(t) == 27);
     REQUIRE(ukkonen::length_maximal_piece_prefix(t, 004000_w) == 2);
     REQUIRE(ukkonen::length_maximal_piece_prefix(t, 45_w) == 1);
     REQUIRE(ukkonen::length_maximal_piece_prefix(t, 0123_w) == 1);
 
-    t.add_word(004_w);
+    ukkonen::add_word(t, 004_w);
     REQUIRE(ukkonen::number_of_distinct_subwords(t) == 27);
     REQUIRE(ukkonen::length_maximal_piece_prefix(t, 004000_w) == 3);
     REQUIRE(ukkonen::length_maximal_piece_prefix(t, "00456789"_w) == 3);
@@ -246,9 +243,9 @@ namespace libsemigroups {
                           "[quick][ukkonen]") {
     Ukkonen t;
     REQUIRE(t.number_of_distinct_words() == 0);
-    t.add_word(012_w);
+    ukkonen::add_word(t, 012_w);
     REQUIRE(t.number_of_distinct_words() == 1);
-    t.add_word(124_w);
+    ukkonen::add_word(t, 124_w);
     REQUIRE(t.number_of_distinct_words() == 2);
 
     REQUIRE(t.nodes().size() == 11);
@@ -274,10 +271,12 @@ namespace libsemigroups {
     REQUIRE(!ukkonen::is_subword(t, 3_w));
     REQUIRE(!ukkonen::is_subword(t, 13_w));
 
-    REQUIRE(std::count_if(
-                cbegin_wislo(5, {}, 00000_w),
-                cend_wislo(5, {}, 00000_w),
-                [&t](word_type const& w) { return ukkonen::is_subword(t, w); })
+    WordRange words;
+    words.alphabet_size(5).min(0).max(5);
+    REQUIRE((words | rx::filter([&t](word_type const& w) {
+               return ukkonen::is_subword(t, w);
+             })
+             | rx::count())
             == 10);
 
     REQUIRE(ukkonen::is_suffix(t, ""_w));
@@ -326,14 +325,13 @@ namespace libsemigroups {
                           "number_of_pieces",
                           "[quick][ukkonen]") {
     Ukkonen t;
-    t.add_word(012_w);
-    t.add_word(0_w);
-    t.add_word(1_w);
-    t.add_word(2_w);
+    ukkonen::add_word(t, 012_w);
+    ukkonen::add_word(t, 0_w);
+    ukkonen::add_word(t, 1_w);
+    ukkonen::add_word(t, 2_w);
 
     REQUIRE(ukkonen::number_of_pieces(t, 012_w) == 3);
-    REQUIRE(ukkonen::pieces(t, 012_w)
-            == std::vector<word_type>({0_w, 1_w, 2_w}));
+    REQUIRE(ukkonen::pieces(t, 012_w) == std::vector({0_w, 1_w, 2_w}));
     REQUIRE(ukkonen::number_of_pieces(t, 0_w) == 1);
     REQUIRE(ukkonen::pieces(t, 0_w) == std::vector<word_type>({0_w}));
     REQUIRE(ukkonen::number_of_pieces(t, 1_w) == 1);
@@ -341,11 +339,11 @@ namespace libsemigroups {
     REQUIRE(ukkonen::number_of_pieces(t, 2_w) == 1);
     REQUIRE(ukkonen::pieces(t, 2_w) == std::vector<word_type>({2_w}));
 
-    t.add_word("01284567"_w);
-    t.add_word(012_w);  // does nothing
-    t.add_word(845_w);
-    t.add_word(56_w);
-    t.add_word(567_w);
+    ukkonen::add_word(t, "01284567"_w);
+    ukkonen::add_word(t, 012_w);  // does nothing
+    ukkonen::add_word(t, 845_w);
+    ukkonen::add_word(t, 56_w);
+    ukkonen::add_word(t, 567_w);
 
     REQUIRE(t.number_of_distinct_words() == 8);
     REQUIRE(t.number_of_words() == 9);
@@ -358,7 +356,7 @@ namespace libsemigroups {
 
     REQUIRE(ukkonen::number_of_pieces(t, "01284567"_w) == 3);
     REQUIRE(ukkonen::pieces(t, "01284567"_w)
-            == std::vector<word_type>({012_w, 845_w, 67_w}));
+            == std::vector({012_w, 845_w, 67_w}));
     REQUIRE(ukkonen::is_piece(t, 012_w));
     REQUIRE(ukkonen::is_piece(t, 845_w));
     REQUIRE(ukkonen::is_piece(t, 67_w));
@@ -373,8 +371,7 @@ namespace libsemigroups {
 
     REQUIRE(ukkonen::pieces(t, 567_w) == std::vector<word_type>({567_w}));
     REQUIRE(ukkonen::number_of_pieces(t, 12845_w) == 2);
-    REQUIRE(ukkonen::pieces(t, 12845_w)
-            == std::vector<word_type>({12_w, 845_w}));
+    REQUIRE(ukkonen::pieces(t, 12845_w) == std::vector({12_w, 845_w}));
 
     auto w = "0128456701284567"_w;
     REQUIRE(ukkonen::pieces(t, w.cbegin(), w.cend()).size() == 7);
@@ -386,9 +383,9 @@ namespace libsemigroups {
   LIBSEMIGROUPS_TEST_CASE("Ukkonen", "005", "traverse", "[quick][ukkonen]") {
     using State = Ukkonen::State;
     Ukkonen t;
-    t.add_word(004000_w);
+    ukkonen::add_word(t, 004000_w);
 
-    auto s = ukkonen::traverse(t, {});
+    auto s = ukkonen::traverse(t, word_type({}));
     REQUIRE(s.first.v == 0);
     REQUIRE(s.first.pos == 0);
 
@@ -424,20 +421,21 @@ namespace libsemigroups {
   LIBSEMIGROUPS_TEST_CASE("Ukkonen", "006", "dot", "[quick][ukkonen]") {
     {
       Ukkonen t;
-      t.add_word(00_w);
-      t.add_word(00_w);
-      t.add_word(010_w);
-      t.add_word(00_w);
-      t.add_word(0101_w);
-      t.add_word(010_w);
+      ukkonen::add_word(t, 00_w);
+      ukkonen::add_word(t, 00_w);
+      ukkonen::add_word(t, 010_w);
+      ukkonen::add_word(t, 00_w);
+      ukkonen::add_word(t, 0101_w);
+      ukkonen::add_word(t, 010_w);
       REQUIRE_NOTHROW(ukkonen::dot(t));
     }
     {
       Ukkonen u;
       // No words
       REQUIRE_THROWS_AS(ukkonen::dot(u), LibsemigroupsException);
-      ukkonen::add_words(
-          u, cbegin_wislo(2, ""_w, "00000"_w), cend_wislo(2, ""_w, "00000"_w));
+      WordRange words;
+      words.alphabet_size(2).min(0).max(5);
+      ukkonen::add_words(u, rx::begin(words), rx::end(words));
       REQUIRE(u.number_of_distinct_words() == 30);
       // Too many words
       REQUIRE_THROWS_AS(ukkonen::dot(u), LibsemigroupsException);
@@ -446,10 +444,10 @@ namespace libsemigroups {
 
   LIBSEMIGROUPS_TEST_CASE("Ukkonen", "007", "strings", "[quick][ukkonen]") {
     Ukkonen t;
-    t.add_word("aaaeaa");
-    t.add_word("abcd");
+    ukkonen::add_word(t, "aaaeaa");
+    ukkonen::add_word(t, "abcd");
     REQUIRE(t.number_of_distinct_words() == 2);
-    t.add_word("");
+    ukkonen::add_word(t, "");
     REQUIRE(t.number_of_distinct_words() == 2);
 
     REQUIRE(t.nodes().size() == 15);
@@ -495,190 +493,15 @@ namespace libsemigroups {
     REQUIRE(ukkonen::number_of_pieces(t, "xxx") == POSITIVE_INFINITY);
   }
 
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "008", "dfs #01", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
+  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "008", "pieces", "[quick][ukkonen]") {
     Ukkonen t;
-    t.add_word(1212_w);
-    t.add_word(0_w);
-    t.add_word(1213121312131213_w);
-    t.add_word(0_w);
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last) == word_type(1213_w));
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "009", "dfs #02", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
-
-    Ukkonen t;
-    t.add_word("aaaaa");
-    t.add_word("bbb");
-    t.add_word("ababa");
-    t.add_word("aaabaabaaabaa");
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last) == word_type({97, 98, 97}));
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "010", "dfs #03", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
-    Ukkonen t;
-    t.add_word("aaaaa");
-    t.add_word("bbb");
-    t.add_word("cba");
-    t.add_word("aaccaca");
-    t.add_word("aba");
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last) == word_type());
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "011", "dfs #04", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
-    Ukkonen t;
-    t.add_word(00_w);
-    t.add_word(10_w);
-    t.add_word(01_w);
-    t.add_word(20_w);
-    t.add_word(02_w);
-    t.add_word(30_w);
-    t.add_word(03_w);
-    t.add_word(11_w);
-    t.add_word(23_w);
-    t.add_word(222_w);
-    t.add_word(12121212121212_w);
-    t.add_word(12131213121312131213121312131213_w);
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last) == 12131213_w);
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "012", "dfs #05", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
-    Ukkonen t;
-    t.add_word(00_w);
-    t.add_word(10_w);
-    t.add_word(01_w);
-    t.add_word(20_w);
-    t.add_word(02_w);
-    t.add_word(30_w);
-    t.add_word(03_w);
-    t.add_word(11_w);
-    t.add_word(23_w);
-    t.add_word(222_w);
-    t.add_word(12121212121212_w);
-    t.add_word(44444444_w);
-    t.add_word(1213_w);
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last) == 12_w);
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "013", "dfs #06", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
-    Ukkonen t;
-    t.add_word(00_w);
-    t.add_word(10_w);
-    t.add_word(01_w);
-    t.add_word(20_w);
-    t.add_word(02_w);
-    t.add_word(30_w);
-    t.add_word(03_w);
-    t.add_word(11_w);
-    t.add_word(23_w);
-    t.add_word(222_w);
-    t.add_word(5555555_w);
-    t.add_word(44444444_w);
-    t.add_word(513_w);
-    t.add_word(12_w);
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last) == 4444_w);
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "014", "dfs #07", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
-    Ukkonen t;
-    t.add_word("aaaaaaaaaaaaaa");
-    t.add_word("bbbbbbbbbbbbbb");
-    t.add_word("cccccccccccccc");
-    t.add_word("aaaaba");
-    t.add_word("bbb");
-    t.add_word("bbbbab");
-    t.add_word("aaa");
-    t.add_word("aaaaca");
-    t.add_word("ccc");
-    t.add_word("ccccac");
-    t.add_word("aaa");
-    t.add_word("bbbbcb");
-    t.add_word("ccc");
-    t.add_word("ccccbc");
-    t.add_word("bbb");
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last) == word_type({99, 99, 99}));
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "015", "dfs #08", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
-    Ukkonen t;
-    t.add_word("aaaaaaaaaaaaaa");
-    t.add_word("bbbbbbbbbbbbbb");
-    t.add_word("ddddcc");
-    t.add_word("aaaaba");
-    t.add_word("bbb");
-    t.add_word("bbbbab");
-    t.add_word("aaa");
-    t.add_word("aaaaca");
-    t.add_word("dcac");
-    t.add_word("aaa");
-    t.add_word("bbbbcb");
-    t.add_word("dcbc");
-    t.add_word("bbb");
-    t.add_word("ccc");
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last) == word_type({98, 98, 98}));
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "016", "dfs #09", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
-    Ukkonen t;
-    t.add_word("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa");
-    t.add_word("bbb");
-    t.add_word("ababa");
-    t.add_word("aaaaaaaaaaaaaaaabaaaabaaaaaaaaaaaaaaaabaaaa");
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last)
-            == word_type({
-                97,
-                97,
-                97,
-                97,
-            }));
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "017", "dfs #10", "[quick][ukkonen]") {
-    using const_iterator = typename Ukkonen::const_iterator;
-    Ukkonen t;
-    t.add_word("aBCbac");
-    t.add_word("bACbaacA");
-    t.add_word("accAABab");
-    const_iterator first, last;
-    std::tie(first, last) = best_subword(t);
-    REQUIRE(word_type(first, last) == word_type());
-  }
-
-  LIBSEMIGROUPS_TEST_CASE("Ukkonen", "018", "pieces", "[quick][ukkonen]") {
-    Ukkonen              t;
-    detail::StringToWord string_to_word("ab");
-    t.add_word(string_to_word("baabbaaaa"));
-    t.add_word(string_to_word("abababbbaa"));
+    ToWord  string_to_word("ab");
+    ukkonen::add_word(t, string_to_word("baabbaaaa"));
+    ukkonen::add_word(t, string_to_word("abababbbaa"));
 
     REQUIRE(ukkonen::number_of_pieces(t, string_to_word("baabbaaaa")) == 3);
     REQUIRE(ukkonen::pieces(t, string_to_word("baabbaaaa"))
-            == std::vector<word_type>({100_w, 1100_w, 00_w}));
+            == std::vector({100_w, 1100_w, 00_w}));
     REQUIRE(ukkonen::is_piece(t, string_to_word("baa")));
     REQUIRE(ukkonen::is_piece(t, string_to_word("bbaa")));
     REQUIRE(ukkonen::is_piece(t, string_to_word("aa")));
@@ -692,25 +515,25 @@ namespace libsemigroups {
 
     REQUIRE(ukkonen::number_of_pieces(t, string_to_word("abababbbaa")) == 3);
     REQUIRE(ukkonen::pieces(t, string_to_word("abababbbaa"))
-            == std::vector<word_type>({0101_w, 011_w, 100_w}));
+            == std::vector({0101_w, 011_w, 100_w}));
   }
 
   LIBSEMIGROUPS_TEST_CASE("Ukkonen",
-                          "019",
+                          "009",
                           "code coverage",
                           "[quick][ukkonen]") {
     Ukkonen u;
-    u.add_word_no_checks(0001000_w);
+    ukkonen::add_word_no_checks(u, 0001000_w);
     auto w = "abcdefabababab";
-    u.add_word_no_checks(w, w + std::strlen(w));
-    u.add_word_no_checks(w);
-    u.add_word_no_checks({1, 2, 3, 4, 0, 0, 1, 1, 0, 0, 1});
+    ukkonen::add_word_no_checks(u, w, w + std::strlen(w));
+    ukkonen::add_word_no_checks(u, w);
+    ukkonen::add_word_no_checks(u, {1, 2, 3, 4, 0, 0, 1, 1, 0, 0, 1});
     std::string ww("abdbadbabdbabdabdj");
-    u.add_word_no_checks(ww);
-    u.add_word(ww);
+    ukkonen::add_word_no_checks(u, ww);
+    ukkonen::add_word(u, ww);
 
     auto www = 0001000_w;
-    u.add_word_no_checks(www.cbegin(), www.cend());
+    ukkonen::add_word_no_checks(u, www.cbegin(), www.cend());
 
     REQUIRE(u.nodes().size() == 78);
     REQUIRE(u.length_of_distinct_words() == 50);
@@ -785,14 +608,14 @@ namespace libsemigroups {
                  5,  10, 4,  9,  3,  8, 2,  7, 6,  5, 4,  3,  2,  1}));
 
     distances.resize(u.number_of_distinct_words(), 0);
-    auto range = IntegralRange<size_t>(0, u.number_of_distinct_words());
+    auto range = detail::IntRange<size_t>(0, u.number_of_distinct_words());
     std::transform(range.cbegin(),
                    range.cend(),
                    distances.begin(),
                    [&u](auto i) { return u.multiplicity(i); });
     REQUIRE(distances == std::vector<size_t>({2, 2, 1, 2}));
 
-    std::vector<word_type> v = {www, www};
+    std::vector v = {www, www};
     ukkonen::add_words_no_checks(u, v);
     ukkonen::add_words(u, v);
     ukkonen::add_words_no_checks(u, v.cbegin(), v.cend());
@@ -801,7 +624,7 @@ namespace libsemigroups {
   }
 
   LIBSEMIGROUPS_TEST_CASE("Ukkonen",
-                          "020",
+                          "010",
                           "code coverage",
                           "[quick][ukkonen]") {
     Ukkonen u;
